/*
 * calccore/adteval.cc
 * 
 * Animation description tree eval code. 
 * This is part of the AniVision project. 
 * 
 * Copyright (c) 2003 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "adtree.h"
#include <ani-parser/exprtf.h>
#include <core/execthread.h>
#include <core/aniinstance.h>
#include <core/anicontext.h>

#include <calccore/ccif.h>


namespace CC
{

int ADTNAniDescBlock::Eval(ANI::ExprValue &nvalue,ANI::ExecThreadInfo *info)
{
	assert(info);
	ExecThread *et=(ExecThread*)info;
	
	if(!et->Context())
	{
		Error(GetLocationRange(),
			"attempt to execute ani desc stmt while not in animation "
			"context\n");
		abort();
	}
	
	// We're normally inside an TNAniDescStmt. In this case, lookup 
	// the desired calc core if specified: 
	RefString core_name;
	ANI::TNIdentifier *name_idf=NULL;
	do {
		if(!parent() || 
			parent()->NType()!=ANI::TN_Statement || 
			((ANI::TNStatement*)parent())->StmtType()!=ANI::TNS_AniDescStmt)
		{  break;  }
		name_idf=((ANI::TNAniDescStmt*)parent())->core_name;
		if(!name_idf)  break;
		core_name=name_idf->CompleteStr();
	} while(0);
	if(core_name)
	{
		ANI::TNLocation name_loc;
		if(name_idf)  name_loc=name_idf->GetLocationRange();
		Error(name_loc,"Unknown core name \"%s\" (ignored)\n",core_name.str());
if(core_name!="scc")
{  assert(!"!implemented");  return(0);  }
	}
	
	// NOTE!!
	// - nvalue (as passed) is NOT the return value but stores the 
	//   value we're animating. 
	// - nvalue is an lvalue (or an object)
	//   [i.e. if it is an object, it need not be an lvalue]
	switch(nvalue.GetExprValueTypeID())
	{
		case ANI::EVT_AniScope:
		{
			ANI::ScopeInstance _inst;
			nvalue.GetAniScope(&_inst);
			AniObjectInstance *inst=(AniObjectInstance*)_inst.GetASIB();
			assert(inst->ASType()==AniScopeBase::AST_Object);
			
			CCIF_Object *o_ccif=inst->GetCCIF();
			assert(o_ccif);
			int rv=o_ccif->EvalAniDescBlock(this,et);
			switch(rv)
			{
				case 1:  return(1);  // thread switch
				case 0:  return(0);  // regular return
				case -1:  // (non-thread-switch) return with error
					#warning "Accumulate errors... FIXME"
					return(0);
				default: assert(0);
			}
		} break;
		case ANI::EVT_POD:
		{
			//ANI::ExprValueType evt;
			//nvalue.GetExprValueType(&evt);
			//const ANI::Value::CompleteType *pod=evt.PODType();
			//assert(pod);
			
			CCIF_Setting *s_ccif=et->Context()->ccif;
			assert(s_ccif);
			fprintf(stderr,"CC: IMPLEMENT ME: Eval ani desc block "
				"for POD (calccore)\n");
		} break;
		default: assert(0);
	}
	
	return(0);
}


}  // end of namespace CC
